﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using EJ2ScheduleSample.Models;
using EJ2ScheduleSample.Data;

namespace EJ2ScheduleSample.Controllers
{
    public class HomeController : Controller
    {
      
        private readonly ScheduleDataContext _context;

        public HomeController(ScheduleDataContext context)
        {
            _context = context;
        }

        // GET: Events
        public IActionResult Index()
        {
            return View();
        }

        [HttpPost]
        public List<ScheduleEvent> LoadData([FromBody]Params param)
        {
            return _context.ScheduleEvents.ToList();
            //return _context.ScheduleEvent.ToList().Where(app => app.StartTime >= param.StartDate && app.StartTime <= param.EndDate).ToList();
        }

        public class Params
        {
            public DateTime StartDate { get; set; }
            public DateTime EndDate { get; set; }
        }

        [HttpPost]
        public List<ScheduleEvent> UpdateData([FromBody]EditParams param)
        {
            if (param.action == "insert" || (param.action == "batch" && param.added.Count > 0))
            {
                var value = (param.action == "insert") ? param.value : param.added[0];
                int intMax = _context.ScheduleEvents.ToList().Count > 0 ? _context.ScheduleEvents.ToList().Max(p => p.Id) : 1;
                DateTime startTime = Convert.ToDateTime(value.StartTime);
                DateTime endTime = Convert.ToDateTime(value.EndTime);
                ScheduleEvent appointment = new ScheduleEvent()
                {
                   
                    StartTime = startTime.ToLocalTime(),
                    EndTime = endTime.ToLocalTime(),
                    Subject = value.Subject,
                    IsAllDay = value.IsAllDay,
                    StartTimezone = value.StartTimezone,
                    EndTimezone = value.EndTimezone,
                    RecurrenceRule = value.RecurrenceRule,
                    RecurrenceId = value.RecurrenceId,
                    RecurrenceException = value.RecurrenceException,
                    Description = value.Description,
                  //  EventType = value.EventType
                };
                _context.ScheduleEvents.Add(appointment);
                _context.SaveChanges();
            }
            if (param.action == "update" || (param.action == "batch" && param.changed.Count > 0))
            {
                var value = (param.action == "update") ? param.value : param.changed[0];
                var filterData = _context.ScheduleEvents.Where(c => c.Id == Convert.ToInt32(value.Id));
                if (filterData.Count() > 0)
                {
                    DateTime startTime = Convert.ToDateTime(value.StartTime);
                    DateTime endTime = Convert.ToDateTime(value.EndTime);
                    ScheduleEvent appointment = _context.ScheduleEvents.Single(A => A.Id == Convert.ToInt32(value.Id));
                    appointment.StartTime = startTime.ToLocalTime();
                    appointment.EndTime = endTime.ToLocalTime();
                    appointment.StartTimezone = value.StartTimezone;
                    appointment.EndTimezone = value.EndTimezone;
                    appointment.Subject = value.Subject;
                    appointment.IsAllDay = value.IsAllDay;
                    appointment.RecurrenceRule = value.RecurrenceRule;
                    appointment.RecurrenceId =  value.RecurrenceId;
                    appointment.RecurrenceException = value.RecurrenceException;
                    appointment.Description = value.Description;
                 //   appointment.EventType = value.EventType;
                }
                _context.SaveChanges();
            }
            if (param.action == "remove" || (param.action == "batch" && param.deleted.Count > 0))
            {
                if (param.action == "remove")
                {
                    int key = Convert.ToInt32(param.key);
                    ScheduleEvent appointment = _context.ScheduleEvents.Where(c => c.Id == key).FirstOrDefault();
                    if (appointment != null) _context.ScheduleEvents.Remove(appointment);
                }
                else
                {
                    foreach (var apps in param.deleted)
                    {
                        ScheduleEvent appointment = _context.ScheduleEvents.Where(c => c.Id == apps.Id).FirstOrDefault();
                        if (apps != null) _context.ScheduleEvents.Remove(appointment);
                    }
                }
                _context.SaveChanges();
            }
            return _context.ScheduleEvents.ToList();
        }




        [HttpPost]
        public JsonResult GetData()
        {
            //ViewBag.appointments = appointments;
            List<ScheduleEvent> data = _context.ScheduleEvents.ToList(); //  .Where(c => c.OwnerId == User.Identity.Name.ToLower()).ToList();
            return Json(data);
            // return datas;
        }
        //public JsonResult Batch([FromBody] EditParams param)
        public JsonResult CrudData(EditParams param)
        {
            // this block of code will execute while inserting the appointments
            if (param.action == "insert" || (param.action == "CrudData" && param.added != null))
            {
                ScheduleEvent appoint = new ScheduleEvent();
                object result;
                if (param.action == "insert")
                {
                    var value = param.value;
                    foreach (var fieldName in value.GetType().GetProperties())
                    {
                        var newName = fieldName.ToString().Split(null);
                        if (newName[1] == "Id") result = (_context.ScheduleEvents.ToList().Count > 0 ? _context.ScheduleEvents.ToList().Max(p => p.Id) : 1) + 1;
                        else if (newName[1] == "StartTime" || newName[1] == "EndTime") result = Convert.ToDateTime(fieldName.GetValue(value));
                        else result = fieldName.GetValue(value);
                        fieldName.SetValue(appoint, result);
                    }
                    _context.ScheduleEvents.Add(appoint);
                }
                else
                {
                    foreach (var item in param.added.Select((x, i) => new { Value = x, Index = i }))
                    {
                        var value = item.Value;
                        foreach (var fieldName in value.GetType().GetProperties())
                        {
                            var newName = fieldName.ToString().Split(null);
                            if (newName[1] == "Id") result = (_context.ScheduleEvents.ToList().Count > 0 ? _context.ScheduleEvents.ToList().Max(p => p.Id) : 1) + 1 + item.Index;
                            else if (newName[1] == "StartTime" || newName[1] == "EndTime") result = Convert.ToDateTime(fieldName.GetValue(value));
                            else result = fieldName.GetValue(value);
                            fieldName.SetValue(appoint, result);
                        }
                        _context.ScheduleEvents.Add(appoint);
                    }
                }
                _context.SaveChanges();
            }
            if ((param.action == "remove") || (param.action == "CrudData" && param.deleted != null)) // this block of code will execute while removing the appointment
            {
                if (param.action == "remove")
                {
                    ScheduleEvent app = _context.ScheduleEvents.Where(c => c.Id == Convert.ToInt32(param.key)).FirstOrDefault();
                    if (app != null) _context.ScheduleEvents.Remove(app);
                }
                else
                {
                    foreach (var a in param.deleted)
                    {
                        var app = _context.ScheduleEvents.ToList().Where(c => c.Id == Convert.ToInt32(a.Id)).FirstOrDefault();
                        if (app != null) _context.ScheduleEvents.Remove(app);
                    }
                }
                _context.SaveChanges();
            }
            if (param.action == "update" || (param.action == "batch" && param.changed != null)) // this block of code will execute while updating the appointment
            {
                var value = param.action == "update" ? param.value : param.changed[0];
                var filterData = _context.ScheduleEvents.Where(c => c.Id == Convert.ToInt32(value.Id));
                if (filterData.Count() > 0)
                {
                    ScheduleEvent appoint = _context.ScheduleEvents.Single(A => A.Id == Convert.ToInt32(value.Id));
                    appoint.StartTime = Convert.ToDateTime(value.StartTime);
                    appoint.EndTime = Convert.ToDateTime(value.EndTime);
                    appoint.Subject = value.Subject;
                    appoint.RecurrenceId = value.RecurrenceId;
                    appoint.IsAllDay = value.IsAllDay;
                    appoint.RecurrenceRule = value.RecurrenceRule;
                    //appoint.OwnerId = User.Identity.Name.ToLower();
                }
                _context.SaveChanges();
            }
            List<ScheduleEvent> data = _context.ScheduleEvents.ToList();// .Where(c => c.OwnerId == User.Identity.Name.ToLower()).ToList();
            return Json(data);
        }

        // CRUD METODO
        //public JsonResult UpdateData(EditParams param)
        //{
        //    // this block of code will execute while inserting the appointments
        //    if (param.action == "insert" || (param.action == "batch" && param.added != null))
        //    {
        //        var value = param.action == "insert" ? param.value : param.added[0];
        //        int intMax = _context.DefaultSchedule.ToList().Count > 0 ? _context.DefaultSchedule.ToList().Max(p => p.Id) : 1;
        //        DateTime startTime = Convert.ToDateTime(value.StartTime);
        //        DateTime endTime = Convert.ToDateTime(value.EndTime);
        //        DefaultSchedule appoint = new DefaultSchedule()
        //        {
        //            Id = intMax + 1,
        //            StartTime = startTime,
        //            EndTime = endTime,
        //            Subject = value.Subject,
        //            AllDay = value.AllDay
        //        };
        //        _context.DefaultSchedule.InsertOnSubmit(appoint);
        //        _context.SubmitChanges();
        //    }
        //    // this block of code will execute while removing the appointment
        //    if (param.action == "remove" || param.deleted != null)
        //    {
        //        if (param.action == "remove")
        //        {
        //            int key = Convert.ToInt32(param.key);
        //            DefaultSchedule app = _context.DefaultSchedule.Where(c => c.Id == key).FirstOrDefault();
        //            if (app != null) _context.DefaultSchedule.DeleteOnSubmit(app);
        //        }
        //        else
        //        {
        //            foreach (var apps in param.deleted)
        //            {
        //                DefaultSchedule app = _context.DefaultSchedule.Where(c => c.Id == apps.Id).FirstOrDefault();
        //                if (apps != null) _context.DefaultSchedule.DeleteOnSubmit(app);
        //            }
        //        }
        //        _context.SubmitChanges();
        //    }
        //    // this block of code will execute while updating the appointment
        //    if ((param.action == "batch" && param.changed != null) || param.action == "update")
        //    {
        //        var value = param.action == "update" ? param.value : param.changed[0];
        //        var filterData = _context.DefaultSchedule.Where(c => c.Id == Convert.ToInt32(value.Id));

        //        if (filterData.Count() > 0)
        //        {
        //            DateTime startTime = Convert.ToDateTime(value.StartTime);
        //            DateTime endTime = Convert.ToDateTime(value.EndTime);
        //            DefaultSchedule appoint = _context.DefaultSchedule.Single(A => A.Id == Convert.ToInt32(value.Id));
        //            appoint.StartTime = startTime;
        //            appoint.EndTime = endTime;
        //            appoint.Subject = value.Subject;
        //            appoint.AllDay = value.AllDay;
        //        }
        //        _context.SubmitChanges();
        //    }
        //    var data = _context.DefaultSchedule.ToList();
        //    return Json(data, JsonRequestBehavior.AllowGet);
        //}
    }
}
